/*
** Copyright (C) 2003-2005 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "dialogcompare.h"
#include "projects.h"
#include "projectutils.h"
#include "directories.h"


CompareDialog::CompareDialog (Configuration * configuration)
{
  // Store variables
  myconfiguration = configuration;
  
  comparedialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (comparedialog), "Compare project");
  gtk_window_set_position (GTK_WINDOW (comparedialog), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_modal (GTK_WINDOW (comparedialog), TRUE);

  dialog_vbox1 = GTK_DIALOG (comparedialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  label1 = gtk_label_new ("Select a project to compare the current project with\n"
                          "and then press OK");
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (vbox1), label1, FALSE, FALSE, 4);
  gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);

  combo1 = gtk_combo_new ();
  gtk_widget_show (combo1);
  gtk_box_pack_start (GTK_BOX (vbox1), combo1, FALSE, FALSE, 4);

  // Get all projects
  Projects projects (directories_get_projects ());

  // Fill the combo box 
  combo1_items = NULL;
  for (unsigned int i = 0; i < projects.projects.size (); i++) {
    combo1_items = g_list_append (combo1_items, (gpointer) projects.projects[i].c_str ());
  }
  gtk_combo_set_popdown_strings (GTK_COMBO (combo1), combo1_items);
  g_list_free (combo1_items);

  combo_entry1 = GTK_COMBO (combo1)->entry;
  gtk_widget_show (combo_entry1);
  gtk_editable_set_editable (GTK_EDITABLE (combo_entry1), FALSE);
  gtk_entry_set_activates_default (GTK_ENTRY (combo_entry1), TRUE);

  // Set the project we last compared with as the default (if present)
  for (unsigned int i = 0; i < projects.projects.size (); i++) {
    if (projects.projects[i] == configuration->project_to_compare_with) {
      gtk_entry_set_text (GTK_ENTRY (combo_entry1), configuration->project_to_compare_with.c_str());
    }
  }
  
  checkbutton1 = gtk_check_button_new_with_mnemonic ("_Print changed verses only");
  gtk_widget_show (checkbutton1);
  gtk_box_pack_start (GTK_BOX (vbox1), checkbutton1, FALSE, FALSE, 0);
  
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton1), myconfiguration->print_changes_only);

  dialog_action_area1 = GTK_DIALOG (comparedialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  cancelbutton = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (comparedialog), cancelbutton, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton, GTK_CAN_DEFAULT);

  okbutton = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (comparedialog), okbutton, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton, GTK_CAN_DEFAULT);
  
  g_signal_connect ((gpointer) okbutton, "clicked", G_CALLBACK (static_on_okbutton_clicked), gpointer (this));

  gtk_widget_grab_focus (combo_entry1);
  gtk_widget_grab_default (okbutton);
}


CompareDialog::~CompareDialog ()
{
  gtk_widget_destroy (comparedialog);
}


int
CompareDialog::run ()
{
  return gtk_dialog_run (GTK_DIALOG (comparedialog));
}


void
CompareDialog::static_on_okbutton_clicked (GtkButton * button, gpointer user_data)
{
  ((CompareDialog *) user_data)->on_okbutton_clicked ();
}


void
CompareDialog::on_okbutton_clicked ()
{
  // Store user entered data.
  myconfiguration->project_to_compare_with = gtk_entry_get_text (GTK_ENTRY (combo_entry1));
  myconfiguration->print_changes_only = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton1));
}
