/*
** Copyright (C) 2003-2005 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "dialogselectstyle.h"
#include "bible.h"
#include "stylesheetutils.h"
#include "usfmtools.h"
#include <gdk/gdkkeysyms.h>


SelectStyleDialog::SelectStyleDialog (const string& stylesheet, const string& style)
{
  selectstyledialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (selectstyledialog), "Stylesheet");
  gtk_window_set_position (GTK_WINDOW (selectstyledialog), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_modal (GTK_WINDOW (selectstyledialog), TRUE);
  gtk_window_set_destroy_with_parent (GTK_WINDOW (selectstyledialog), TRUE);
  gtk_window_set_skip_taskbar_hint (GTK_WINDOW (selectstyledialog), TRUE);
  gtk_window_set_skip_pager_hint (GTK_WINDOW (selectstyledialog), TRUE);
  gtk_window_set_type_hint (GTK_WINDOW (selectstyledialog), GDK_WINDOW_TYPE_HINT_DIALOG);

  dialog_vbox1 = GTK_DIALOG (selectstyledialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  vbox2 = gtk_vbox_new (FALSE, 6);
  gtk_widget_show (vbox2);
  gtk_box_pack_start (GTK_BOX (vbox1), vbox2, TRUE, TRUE, 0);

  table2 = gtk_table_new (2, 16, FALSE);
  gtk_widget_show (table2);
  gtk_box_pack_start (GTK_BOX (vbox2), table2, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (table2), 1);
  gtk_table_set_row_spacings (GTK_TABLE (table2), 3);

  // Stores to hold all information.
  store_identification = gtk_list_store_new (1, G_TYPE_STRING);
  store_introduction = gtk_list_store_new (1, G_TYPE_STRING);
  store_introduction2 = gtk_list_store_new (1, G_TYPE_STRING);
  store_titles = gtk_list_store_new (1, G_TYPE_STRING);
  store_titles2 = gtk_list_store_new (1, G_TYPE_STRING);
  store_chapters_verses = gtk_list_store_new (1, G_TYPE_STRING);
  store_paragraphs = gtk_list_store_new (1, G_TYPE_STRING);
  store_poetry = gtk_list_store_new (1, G_TYPE_STRING);
  store_tables = gtk_list_store_new (1, G_TYPE_STRING);
  store_footnotes = gtk_list_store_new (1, G_TYPE_STRING);
  store_crossreference = gtk_list_store_new (1, G_TYPE_STRING);
  store_study = gtk_list_store_new (1, G_TYPE_STRING);
  store_special_styles = gtk_list_store_new (1, G_TYPE_STRING);
  store_special_styles2 = gtk_list_store_new (1, G_TYPE_STRING);
  store_peripheral = gtk_list_store_new (1, G_TYPE_STRING);
  store_other = gtk_list_store_new (1, G_TYPE_STRING);

  treeview_identification = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store_identification));
  gtk_widget_show (treeview_identification);
  gtk_table_attach (GTK_TABLE (table2), treeview_identification, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_set_size_request (treeview_identification, 35, -1);
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview_identification), FALSE);
  gtk_tree_view_set_enable_search (GTK_TREE_VIEW (treeview_identification), FALSE);

  treeview_introduction = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store_introduction));
  gtk_widget_show (treeview_introduction);
  gtk_table_attach (GTK_TABLE (table2), treeview_introduction, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_set_size_request (treeview_introduction, 35, -1);
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview_introduction), FALSE);
  gtk_tree_view_set_enable_search (GTK_TREE_VIEW (treeview_introduction), FALSE);

  treeview_introduction2 = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store_introduction2));
  gtk_widget_show (treeview_introduction2);
  gtk_table_attach (GTK_TABLE (table2), treeview_introduction2, 2, 3, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_set_size_request (treeview_introduction2, 35, -1);
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview_introduction2), FALSE);
  gtk_tree_view_set_enable_search (GTK_TREE_VIEW (treeview_introduction2), FALSE);

  treeview_titles = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store_titles));
  gtk_widget_show (treeview_titles);
  gtk_table_attach (GTK_TABLE (table2), treeview_titles, 3, 4, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_set_size_request (treeview_titles, 35, -1);
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview_titles), FALSE);
  gtk_tree_view_set_enable_search (GTK_TREE_VIEW (treeview_titles), FALSE);

  treeview_titles2 = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store_titles2));
  gtk_widget_show (treeview_titles2);
  gtk_table_attach (GTK_TABLE (table2), treeview_titles2, 4, 5, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_set_size_request (treeview_titles2, 35, -1);
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview_titles2), FALSE);
  gtk_tree_view_set_enable_search (GTK_TREE_VIEW (treeview_titles2), FALSE);
  
  treeview_chapters_verses = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store_chapters_verses));
  gtk_widget_show (treeview_chapters_verses);
  gtk_table_attach (GTK_TABLE (table2), treeview_chapters_verses, 5, 6, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_set_size_request (treeview_chapters_verses, 35, -1);
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview_chapters_verses), FALSE);
  gtk_tree_view_set_enable_search (GTK_TREE_VIEW (treeview_chapters_verses), FALSE);

  treeview_paragraphs = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store_paragraphs));
  gtk_widget_show (treeview_paragraphs);
  gtk_table_attach (GTK_TABLE (table2), treeview_paragraphs, 6, 7, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_set_size_request (treeview_paragraphs, 35, -1);
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview_paragraphs), FALSE);
  gtk_tree_view_set_enable_search (GTK_TREE_VIEW (treeview_paragraphs), FALSE);

  treeview_poetry = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store_poetry));
  gtk_widget_show (treeview_poetry);
  gtk_table_attach (GTK_TABLE (table2), treeview_poetry, 7, 8, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_set_size_request (treeview_poetry, 35, -1);
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview_poetry), FALSE);
  gtk_tree_view_set_enable_search (GTK_TREE_VIEW (treeview_poetry), FALSE);

  treeview_tables = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store_tables));
  gtk_widget_show (treeview_tables);
  gtk_table_attach (GTK_TABLE (table2), treeview_tables, 8, 9, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_set_size_request (treeview_tables, 35, -1);
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview_tables), FALSE);
  gtk_tree_view_set_enable_search (GTK_TREE_VIEW (treeview_tables), FALSE);

  treeview_footnotes = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store_footnotes));
  gtk_widget_show (treeview_footnotes);
  gtk_table_attach (GTK_TABLE (table2), treeview_footnotes, 9, 10, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_set_size_request (treeview_footnotes, 35, -1);
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview_footnotes), FALSE);
  gtk_tree_view_set_enable_search (GTK_TREE_VIEW (treeview_footnotes), FALSE);

  treeview_crossreference = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store_crossreference));
  gtk_widget_show (treeview_crossreference);
  gtk_table_attach (GTK_TABLE (table2), treeview_crossreference, 10, 11, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_set_size_request (treeview_crossreference, 35, -1);
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview_crossreference), FALSE);
  gtk_tree_view_set_enable_search (GTK_TREE_VIEW (treeview_crossreference), FALSE);

  treeview_study = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store_study));
  gtk_widget_show (treeview_study);
  gtk_table_attach (GTK_TABLE (table2), treeview_study, 11, 12, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_set_size_request (treeview_study, 35, -1);
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview_study), FALSE);
  gtk_tree_view_set_enable_search (GTK_TREE_VIEW (treeview_study), FALSE);

  treeview_special_styles = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store_special_styles));
  gtk_widget_show (treeview_special_styles);
  gtk_table_attach (GTK_TABLE (table2), treeview_special_styles, 12, 13, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_set_size_request (treeview_special_styles, 35, -1);
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview_special_styles), FALSE);
  gtk_tree_view_set_enable_search (GTK_TREE_VIEW (treeview_special_styles), FALSE);

  treeview_special_styles2 = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store_special_styles2));
  gtk_widget_show (treeview_special_styles2);
  gtk_table_attach (GTK_TABLE (table2), treeview_special_styles2, 13, 14, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_set_size_request (treeview_special_styles2, 35, -1);
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview_special_styles2), FALSE);
  gtk_tree_view_set_enable_search (GTK_TREE_VIEW (treeview_special_styles2), FALSE);

  treeview_peripheral = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store_peripheral));
  gtk_widget_show (treeview_peripheral);
  gtk_table_attach (GTK_TABLE (table2), treeview_peripheral, 14, 15, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_set_size_request (treeview_peripheral, 35, -1);
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview_peripheral), FALSE);
  gtk_tree_view_set_enable_search (GTK_TREE_VIEW (treeview_peripheral), FALSE);

  treeview_other = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store_other));
  gtk_widget_show (treeview_other);
  gtk_table_attach (GTK_TABLE (table2), treeview_other, 15, 16, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_FILL), 0, 0);
  gtk_widget_set_size_request (treeview_other, 35, -1);
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview_other), FALSE);
  gtk_tree_view_set_enable_search (GTK_TREE_VIEW (treeview_other), FALSE);

  label33 = gtk_label_new ("Identifi-\ncation\nInfor-\nmation");
  gtk_widget_show (label33);
  gtk_table_attach (GTK_TABLE (table2), label33, 0, 1, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (GTK_FILL), 1, 0);
  gtk_misc_set_alignment (GTK_MISC (label33), 0, 0);

  label34 = gtk_label_new ("Introduction");
  gtk_widget_show (label34);
  gtk_table_attach (GTK_TABLE (table2), label34, 1, 3, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (GTK_FILL), 1, 0);
  gtk_misc_set_alignment (GTK_MISC (label34), 0, 0);

  label35 = gtk_label_new ("Title\nHeadings\nand\nLabels");
  gtk_widget_show (label35);
  gtk_table_attach (GTK_TABLE (table2), label35, 3, 5, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (GTK_FILL), 1, 0);
  gtk_misc_set_alignment (GTK_MISC (label35), 0, 0);

  label36 = gtk_label_new ("Chap-\nters\nand\nVerses");
  gtk_widget_show (label36);
  gtk_table_attach (GTK_TABLE (table2), label36, 5, 6, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (GTK_FILL), 1, 0);
  gtk_misc_set_alignment (GTK_MISC (label36), 0, 0);

  label37 = gtk_label_new ("Para-\ngraph");
  gtk_widget_show (label37);
  gtk_table_attach (GTK_TABLE (table2), label37, 6, 7, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (GTK_FILL), 1, 0);
  gtk_misc_set_alignment (GTK_MISC (label37), 0, 0);

  label38 = gtk_label_new ("Poetry");
  gtk_widget_show (label38);
  gtk_table_attach (GTK_TABLE (table2), label38, 7, 8, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (GTK_FILL), 1, 0);
  gtk_misc_set_alignment (GTK_MISC (label38), 0, 0);

  label45 = gtk_label_new ("Table");
  gtk_widget_show (label45);
  gtk_table_attach (GTK_TABLE (table2), label45, 8, 9, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (GTK_FILL), 1, 0);
  gtk_misc_set_alignment (GTK_MISC (label45), 0, 0);

  label39 = gtk_label_new ("Foot-\nnotes");
  gtk_widget_show (label39);
  gtk_table_attach (GTK_TABLE (table2), label39, 9, 10, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (GTK_FILL), 1, 0);
  gtk_misc_set_alignment (GTK_MISC (label39), 0, 0);

  label40 = gtk_label_new ("Cross\nRefe-\nrences");
  gtk_widget_show (label40);
  gtk_table_attach (GTK_TABLE (table2), label40, 10, 11, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (GTK_FILL), 1, 0);
  gtk_misc_set_alignment (GTK_MISC (label40), 0, 0);

  label41 = gtk_label_new ("Exten-\nded\nStudy\nNotes");
  gtk_widget_show (label41);
  gtk_table_attach (GTK_TABLE (table2), label41, 11, 12, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (GTK_FILL), 1, 0);
  gtk_misc_set_alignment (GTK_MISC (label41), 0, 0);

  label42 = gtk_label_new ("Special\nText\nand\nCharacter\nStyles");
  gtk_widget_show (label42);
  gtk_table_attach (GTK_TABLE (table2), label42, 12, 14, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (GTK_FILL), 1, 0);
  gtk_misc_set_alignment (GTK_MISC (label42), 0, 0);

  label43 = gtk_label_new ("Peri-\npheral\nMate-\nrials");
  gtk_widget_show (label43);
  gtk_table_attach (GTK_TABLE (table2), label43, 14, 15, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (GTK_FILL), 1, 0);
  gtk_misc_set_alignment (GTK_MISC (label43), 0, 0);

  label44 = gtk_label_new ("Other");
  gtk_widget_show (label44);
  gtk_table_attach (GTK_TABLE (table2), label44, 15, 16, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (GTK_FILL), 1, 0);
  gtk_misc_set_alignment (GTK_MISC (label44), 0, 0);

  labelname = gtk_label_new ("");
  gtk_widget_show (labelname);
  gtk_box_pack_start (GTK_BOX (vbox2), labelname, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (labelname), 0, 0.5);

  scrolledwindow14 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow14);
  gtk_box_pack_start (GTK_BOX (vbox2), scrolledwindow14, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow14), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

  textview1 = gtk_text_view_new ();
  gtk_widget_show (textview1);
  gtk_container_add (GTK_CONTAINER (scrolledwindow14), textview1);
  gtk_text_view_set_editable (GTK_TEXT_VIEW (textview1), FALSE);
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (textview1), GTK_WRAP_WORD);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (textview1), FALSE);

  dialog_action_area1 = GTK_DIALOG (selectstyledialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  cancelbutton1 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (selectstyledialog), cancelbutton1, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton1, GTK_CAN_DEFAULT);

  okbutton = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (selectstyledialog), okbutton, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton, GTK_CAN_DEFAULT);

  // Text cell renderer.
  GtkCellRenderer *renderer;
  renderer = gtk_cell_renderer_text_new ();

  // Make data visible in the treeviews.
  column_identification = gtk_tree_view_column_new_with_attributes ("", renderer, "text", 0, NULL);
  column_introduction = gtk_tree_view_column_new_with_attributes ("", renderer, "text", 0, NULL);
  column_introduction2 = gtk_tree_view_column_new_with_attributes ("", renderer, "text", 0, NULL);
  column_titles = gtk_tree_view_column_new_with_attributes ("", renderer, "text", 0, NULL);
  column_titles2 = gtk_tree_view_column_new_with_attributes ("", renderer, "text", 0, NULL);
  column_chapters_verses = gtk_tree_view_column_new_with_attributes ("", renderer, "text", 0, NULL);
  column_paragraphs = gtk_tree_view_column_new_with_attributes ("", renderer, "text", 0, NULL);
  column_poetry = gtk_tree_view_column_new_with_attributes ("", renderer, "text", 0, NULL);
  column_tables = gtk_tree_view_column_new_with_attributes ("", renderer, "text", 0, NULL);
  column_footnotes = gtk_tree_view_column_new_with_attributes ("", renderer, "text", 0, NULL);
  column_crossreference = gtk_tree_view_column_new_with_attributes ("", renderer, "text", 0, NULL);
  column_study = gtk_tree_view_column_new_with_attributes ("", renderer, "text", 0, NULL);
  column_special_styles = gtk_tree_view_column_new_with_attributes ("", renderer, "text", 0, NULL);
  column_special_styles2 = gtk_tree_view_column_new_with_attributes ("", renderer, "text", 0, NULL);
  column_peripheral = gtk_tree_view_column_new_with_attributes ("", renderer, "text", 0, NULL);
  column_other = gtk_tree_view_column_new_with_attributes ("", renderer, "text", 0, NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (treeview_identification), column_identification);
  gtk_tree_view_append_column (GTK_TREE_VIEW (treeview_introduction), column_introduction);
  gtk_tree_view_append_column (GTK_TREE_VIEW (treeview_introduction2), column_introduction2);
  gtk_tree_view_append_column (GTK_TREE_VIEW (treeview_titles), column_titles);
  gtk_tree_view_append_column (GTK_TREE_VIEW (treeview_titles2), column_titles2);
  gtk_tree_view_append_column (GTK_TREE_VIEW (treeview_chapters_verses), column_chapters_verses);
  gtk_tree_view_append_column (GTK_TREE_VIEW (treeview_paragraphs), column_paragraphs);
  gtk_tree_view_append_column (GTK_TREE_VIEW (treeview_poetry), column_poetry);
  gtk_tree_view_append_column (GTK_TREE_VIEW (treeview_tables), column_tables);
  gtk_tree_view_append_column (GTK_TREE_VIEW (treeview_footnotes), column_footnotes);
  gtk_tree_view_append_column (GTK_TREE_VIEW (treeview_crossreference), column_crossreference);
  gtk_tree_view_append_column (GTK_TREE_VIEW (treeview_study), column_study);
  gtk_tree_view_append_column (GTK_TREE_VIEW (treeview_special_styles), column_special_styles);
  gtk_tree_view_append_column (GTK_TREE_VIEW (treeview_special_styles2), column_special_styles2);
  gtk_tree_view_append_column (GTK_TREE_VIEW (treeview_peripheral), column_peripheral);
  gtk_tree_view_append_column (GTK_TREE_VIEW (treeview_other), column_other);

  // Set up the selection for each of the tree views.
  select_identification = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview_identification));
  select_introduction = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview_introduction));
  select_introduction2 = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview_introduction2));
  select_titles = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview_titles));
  select_titles2 = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview_titles2));
  select_chapters_verses = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview_chapters_verses));
  select_paragraphs = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview_paragraphs));
  select_poetry = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview_poetry));
  select_tables = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview_tables));
  select_footnotes = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview_footnotes));
  select_crossreference = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview_crossreference));
  select_study = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview_study));
  select_special_styles = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview_special_styles));
  select_special_styles2 = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview_special_styles2));
  select_peripheral = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview_peripheral));
  select_other = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview_other));
  gtk_tree_selection_set_mode (select_identification, GTK_SELECTION_SINGLE);
  gtk_tree_selection_set_mode (select_introduction, GTK_SELECTION_SINGLE);
  gtk_tree_selection_set_mode (select_introduction2, GTK_SELECTION_SINGLE);
  gtk_tree_selection_set_mode (select_titles, GTK_SELECTION_SINGLE);
  gtk_tree_selection_set_mode (select_titles2, GTK_SELECTION_SINGLE);
  gtk_tree_selection_set_mode (select_chapters_verses, GTK_SELECTION_SINGLE);
  gtk_tree_selection_set_mode (select_paragraphs, GTK_SELECTION_SINGLE);
  gtk_tree_selection_set_mode (select_poetry, GTK_SELECTION_SINGLE);
  gtk_tree_selection_set_mode (select_tables, GTK_SELECTION_SINGLE);
  gtk_tree_selection_set_mode (select_footnotes, GTK_SELECTION_SINGLE);
  gtk_tree_selection_set_mode (select_crossreference, GTK_SELECTION_SINGLE);
  gtk_tree_selection_set_mode (select_study, GTK_SELECTION_SINGLE);
  gtk_tree_selection_set_mode (select_special_styles, GTK_SELECTION_SINGLE);
  gtk_tree_selection_set_mode (select_special_styles2, GTK_SELECTION_SINGLE);
  gtk_tree_selection_set_mode (select_peripheral, GTK_SELECTION_SINGLE);
  gtk_tree_selection_set_mode (select_other, GTK_SELECTION_SINGLE);
  
  // Get all available styles and sort them out.
  stylesheet_get_styles (stylesheet, styles);
  vector<string> markers;
  for (unsigned int i = 0; i < styles.size(); i++)
    markers.push_back (styles[i].marker);
  vector<UsfmCategories> categories;
  usfm_categorize_markers (markers, categories);
  // Add the data to the stores.
  GtkTreeIter iter;
  GtkListStore *store;
  GtkTreeSelection *selection;
  for (unsigned int i = 0; i < markers.size(); i++) {
    store = get_store (categories[i]);
    selection = get_selection (categories[i]);
    // Add the marker to the store.
    gtk_list_store_append (store, &iter);
    gtk_list_store_set (store, &iter, 0, markers[i].c_str (), -1);
  }
  // Select the marker we wish to start off with.
  select_marker (style);
  
  g_signal_connect ((gpointer) treeview_identification, "button_press_event", G_CALLBACK (on_treeview_button_press_event), gpointer(this));
  g_signal_connect ((gpointer) treeview_identification, "key_press_event", G_CALLBACK (on_treeview_key_press_event), gpointer(this));
  g_signal_connect ((gpointer) treeview_introduction, "button_press_event", G_CALLBACK (on_treeview_button_press_event), gpointer(this));
  g_signal_connect ((gpointer) treeview_introduction, "key_press_event", G_CALLBACK (on_treeview_key_press_event), gpointer(this));
  g_signal_connect ((gpointer) treeview_introduction2, "button_press_event", G_CALLBACK (on_treeview_button_press_event), gpointer(this));
  g_signal_connect ((gpointer) treeview_introduction2, "key_press_event", G_CALLBACK (on_treeview_key_press_event), gpointer(this));
  g_signal_connect ((gpointer) treeview_titles, "button_press_event", G_CALLBACK (on_treeview_button_press_event), gpointer(this));
  g_signal_connect ((gpointer) treeview_titles, "key_press_event", G_CALLBACK (on_treeview_key_press_event), gpointer(this));
  g_signal_connect ((gpointer) treeview_titles2, "button_press_event", G_CALLBACK (on_treeview_button_press_event), gpointer(this));
  g_signal_connect ((gpointer) treeview_titles2, "key_press_event", G_CALLBACK (on_treeview_key_press_event), gpointer(this));
  g_signal_connect ((gpointer) treeview_chapters_verses, "button_press_event", G_CALLBACK (on_treeview_button_press_event), gpointer(this));
  g_signal_connect ((gpointer) treeview_chapters_verses, "key_press_event", G_CALLBACK (on_treeview_key_press_event), gpointer(this));
  g_signal_connect ((gpointer) treeview_paragraphs, "button_press_event", G_CALLBACK (on_treeview_button_press_event), gpointer(this));
  g_signal_connect ((gpointer) treeview_paragraphs, "key_press_event", G_CALLBACK (on_treeview_key_press_event), gpointer(this));
  g_signal_connect ((gpointer) treeview_poetry, "button_press_event", G_CALLBACK (on_treeview_button_press_event), gpointer(this));
  g_signal_connect ((gpointer) treeview_poetry, "key_press_event", G_CALLBACK (on_treeview_key_press_event), gpointer(this));
  g_signal_connect ((gpointer) treeview_tables, "button_press_event", G_CALLBACK (on_treeview_button_press_event), gpointer(this));
  g_signal_connect ((gpointer) treeview_tables, "key_press_event", G_CALLBACK (on_treeview_key_press_event), gpointer(this));
  g_signal_connect ((gpointer) treeview_footnotes, "button_press_event", G_CALLBACK (on_treeview_button_press_event), gpointer(this));
  g_signal_connect ((gpointer) treeview_footnotes, "key_press_event", G_CALLBACK (on_treeview_key_press_event), gpointer(this));
  g_signal_connect ((gpointer) treeview_crossreference, "button_press_event", G_CALLBACK (on_treeview_button_press_event), gpointer(this));
  g_signal_connect ((gpointer) treeview_crossreference, "key_press_event", G_CALLBACK (on_treeview_key_press_event), gpointer(this));
  g_signal_connect ((gpointer) treeview_study, "button_press_event", G_CALLBACK (on_treeview_button_press_event), gpointer(this));
  g_signal_connect ((gpointer) treeview_study, "key_press_event", G_CALLBACK (on_treeview_key_press_event), gpointer(this));
  g_signal_connect ((gpointer) treeview_special_styles, "button_press_event", G_CALLBACK (on_treeview_button_press_event), gpointer(this));
  g_signal_connect ((gpointer) treeview_special_styles, "key_press_event", G_CALLBACK (on_treeview_key_press_event), gpointer(this));
  g_signal_connect ((gpointer) treeview_special_styles2, "button_press_event", G_CALLBACK (on_treeview_button_press_event), gpointer(this));
  g_signal_connect ((gpointer) treeview_special_styles2, "key_press_event", G_CALLBACK (on_treeview_key_press_event), gpointer(this));
  g_signal_connect ((gpointer) treeview_peripheral, "button_press_event", G_CALLBACK (on_treeview_button_press_event), gpointer(this));
  g_signal_connect ((gpointer) treeview_peripheral, "key_press_event", G_CALLBACK (on_treeview_key_press_event), gpointer(this));
  g_signal_connect ((gpointer) treeview_other, "button_press_event", G_CALLBACK (on_treeview_button_press_event), gpointer(this));
  g_signal_connect ((gpointer) treeview_other, "key_press_event", G_CALLBACK (on_treeview_key_press_event), gpointer(this));
  g_signal_connect ((gpointer) okbutton, "clicked", G_CALLBACK (on_okbutton_clicked), gpointer (this));

  // Focus must not be grabbed here, because the listview that has the selected
  // style already has grabbed it. Grab the default only.
  gtk_widget_grab_default (okbutton);
  
  update_gui ();
}


SelectStyleDialog::~SelectStyleDialog ()
{
  gtk_widget_destroy (selectstyledialog);
}


int
SelectStyleDialog::run ()
{
  return gtk_dialog_run (GTK_DIALOG (selectstyledialog));
}



void SelectStyleDialog::on_ok_clicked ()
{
  // Store the selected marker.
  get_selected_markers ();
  if (currently_selected_markers.size() > 0)
    selected_style = currently_selected_markers[0];
}


void SelectStyleDialog::get_selected_markers ()
{
  // Clear storage.
  currently_selected_markers.clear();
  // Get, through callbacks, the selected marker.
  gtk_tree_selection_selected_foreach (select_identification, selection_foreach_function, gpointer(this));
  gtk_tree_selection_selected_foreach (select_introduction, selection_foreach_function, gpointer(this));
  gtk_tree_selection_selected_foreach (select_introduction2, selection_foreach_function, gpointer(this));
  gtk_tree_selection_selected_foreach (select_titles, selection_foreach_function, gpointer(this));
  gtk_tree_selection_selected_foreach (select_titles2, selection_foreach_function, gpointer(this));
  gtk_tree_selection_selected_foreach (select_chapters_verses, selection_foreach_function, gpointer(this));
  gtk_tree_selection_selected_foreach (select_paragraphs, selection_foreach_function, gpointer(this));
  gtk_tree_selection_selected_foreach (select_poetry, selection_foreach_function, gpointer(this));
  gtk_tree_selection_selected_foreach (select_tables, selection_foreach_function, gpointer(this));
  gtk_tree_selection_selected_foreach (select_footnotes, selection_foreach_function, gpointer(this));
  gtk_tree_selection_selected_foreach (select_crossreference, selection_foreach_function, gpointer(this));
  gtk_tree_selection_selected_foreach (select_study, selection_foreach_function, gpointer(this));
  gtk_tree_selection_selected_foreach (select_special_styles, selection_foreach_function, gpointer(this));
  gtk_tree_selection_selected_foreach (select_special_styles2, selection_foreach_function, gpointer(this));
  gtk_tree_selection_selected_foreach (select_peripheral, selection_foreach_function, gpointer(this));
  gtk_tree_selection_selected_foreach (select_other, selection_foreach_function, gpointer(this));
  // Store one selected marker.
  currently_selected_marker.clear();
  if (currently_selected_markers.size() > 0)
    currently_selected_marker = currently_selected_markers[0];
}


void SelectStyleDialog::selection_foreach_function (GtkTreeModel * model,
                        GtkTreePath * path, GtkTreeIter * iter, gpointer data)
{
  gchar *marker;
  gtk_tree_model_get (model, iter, 0, &marker, -1);
  ((SelectStyleDialog *) data)->currently_selected_markers.push_back (marker);
  g_free (marker);
}


void SelectStyleDialog::on_okbutton_clicked (GtkButton * button, gpointer user_data)
{
  ((SelectStyleDialog *) user_data)->on_ok_clicked ();
}


gboolean SelectStyleDialog::on_treeview_button_press_event (GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
  return ((SelectStyleDialog *) user_data)->on_treeview_button (widget, event);
}


gboolean SelectStyleDialog::on_treeview_button (GtkWidget *widget, GdkEventButton *event)
{
  if (event->type == GDK_2BUTTON_PRESS) {
    on_ok_clicked ();
    gtk_dialog_response (GTK_DIALOG (selectstyledialog), GTK_RESPONSE_OK);
  }
  if (event->type == GDK_BUTTON_PRESS) {
    update_gui_event = ugeClick;
    g_timeout_add (1, GSourceFunc (on_update_gui), gpointer (this));
  }
  return false;
}


gboolean SelectStyleDialog::on_treeview_key_press_event (GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
  return ((SelectStyleDialog *) user_data)->on_treeview_key (widget, event);
}


gboolean SelectStyleDialog::on_treeview_key (GtkWidget * widget, GdkEventKey * event)
{
  // Pressing Enter (note: there are two Enter keys) closes dialog.
  if (event->keyval == GDK_Return || event->keyval == GDK_KP_Enter) {
    on_ok_clicked ();
    gtk_dialog_response (GTK_DIALOG (selectstyledialog), GTK_RESPONSE_OK);
  }
  // Pressing right or left arrow selects next or previous row.
  if (event->keyval == GDK_Left || event->keyval == GDK_KP_Left) {
    update_gui_event = ugeLeft;
    g_timeout_add (1, GSourceFunc (on_update_gui), gpointer (this));
  }
  if (event->keyval == GDK_Right || event->keyval == GDK_KP_Right) {
    update_gui_event = ugeRight;
    g_timeout_add (1, GSourceFunc (on_update_gui), gpointer (this));
  }
  if (event->keyval == GDK_Up || event->keyval == GDK_KP_Up || event->keyval == GDK_Down || event->keyval == GDK_KP_Down) {
    update_gui_event = ugeUpDown;
    g_timeout_add (1, GSourceFunc (on_update_gui), gpointer (this));
  }
  return false;
}


bool SelectStyleDialog::on_update_gui (gpointer data)
{
  ((SelectStyleDialog *) data)->update_gui ();
  return false;
}


void SelectStyleDialog::update_gui ()
{
  // Take action depending on what event we have.
  switch (update_gui_event) {
    case ugeUpDown :
    {
      // Up or down arrow was pressed: no special action here.
      break;
    }
    case ugeLeft : // Fall through to next, because left and right are similar.
    case ugeRight :
    {
      // Get the marker that has been selected, and get its store.
      get_selected_markers ();
      vector<string> markers;
      vector<UsfmCategories> categories;
      markers.push_back (currently_selected_marker);
      usfm_categorize_markers (markers, categories);
      UsfmCategories category = categories[0];
      GtkListStore * store;
      store = get_store (category);
      // Get the height of the currently selected marker in its store.
      int current_height = 0;
      get_markers (GTK_TREE_MODEL (store), markers);
      for (unsigned int i = 0; i < markers.size(); i++) {
        if (currently_selected_marker == markers[i])
          current_height = i;
      }
      // Keep getting the next or previous store until there are either no 
      // stores left, or the store contains data.
      bool keep_going;
      keep_going = get_next_category (category, update_gui_event == ugeRight);
      while (keep_going) {
        // Next/previous category is available, see whether the store has data.
        store = get_store (category);
        get_markers (GTK_TREE_MODEL (store), markers);
        if (markers.size() > 0) {
          // If a new store is found, get the marker on the same height as the
          // currently selected marker, or if there are not so many markers in it,
          // get the highest marker available. Select that marker.
          unsigned int height = current_height;
          height = CLAMP (height, 0, markers.size() - 1);
          select_marker (markers[height]);
          break;
        }
        keep_going = get_next_category (category, update_gui_event == ugeRight);
      }
      break;
    }
    case ugeClick :
    {
      // Find the new marker.
      get_selected_markers();
      string newmarker;
      for (unsigned int i = 0; i < currently_selected_markers.size(); i++) {
        if (currently_selected_markers[i] != previously_selected_marker) {
          newmarker = currently_selected_markers[i];
        }
      }
      // If new marker was found, select it.
      if (newmarker.length() > 0)
        select_marker (newmarker);
      break;
    }
  }
  // Show name and description of the selected marker.
  get_selected_markers();
  for (unsigned int i = 0; i < styles.size(); i++) {
    if (currently_selected_marker == styles[i].marker) {
      gtk_label_set_text (GTK_LABEL (labelname), styles[i].name.c_str());
      gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview1)), styles[i].info.c_str(), -1);
    }
  }
  // Store selected marker.
  previously_selected_marker = currently_selected_marker;
}


GtkTreeSelection * SelectStyleDialog::get_selection (UsfmCategories category)
// Returns a pointer to the GtkTreeSelection belonging to category.
{
  GtkTreeSelection *selection;
  switch (category) {
    case ucIdentificationInformation :
    {
      selection = select_identification;
      break;
    }
    case ucIntroductionElements :
    {
      selection = select_introduction;
      break;
    } 
    case ucIntroductionElements2 :
    {
      selection = select_introduction2;
      break;
    } 
    case ucTitlesHeadingsAndLabels :
    {
      selection = select_titles;
      break;
    }
    case ucTitlesHeadingsAndLabels2 :
    {
      selection = select_titles2;
      break;
    }
    case ucChaptersAndVerses :
    {
      selection = select_chapters_verses;
      break;
    } 
    case ucParagraphElements :
    {
      selection = select_paragraphs;
      break;
    } 
    case ucPoetryElements :
    {
      selection = select_poetry;
      break;
    } 
    case ucTableElements :
    {
      selection = select_tables;
      break;
    }
    case ucFootnotes :
    {
      selection = select_footnotes;
      break;
    } 
    case ucCrossReferences :
    {
      selection = select_crossreference;
      break;
    } 
    case ucExtendedStudyNotes :
    {
      selection = select_study;
      break;
    } 
    case ucSpecialTextAndCharacterStyles :
    {
      selection = select_special_styles;
      break;
    }
    case ucSpecialTextAndCharacterStyles2 :
    {
      selection = select_special_styles2;
      break;
    }
    case ucPeripheralMaterials :
    {
      selection = select_peripheral;
      break;
    } 
    case ucOtherElements :
    {
      selection = select_other;
      break;
    }
  }
  return selection;
}


GtkListStore * SelectStyleDialog::get_store (UsfmCategories category)
// Returns a pointer to the GtkListStore belonging to category.
{
  GtkListStore *store;
  switch (category) {
    case ucIdentificationInformation :
    {
      store = store_identification;
      break;
    }
    case ucIntroductionElements :
    {
      store = store_introduction;
      break;
    } 
    case ucIntroductionElements2 :
    {
      store = store_introduction2;
      break;
    } 
    case ucTitlesHeadingsAndLabels :
    {
      store = store_titles;
      break;
    }
    case ucTitlesHeadingsAndLabels2 :
    {
      store = store_titles2;
      break;
    }
    case ucChaptersAndVerses :
    {
      store = store_chapters_verses;
      break;
    } 
    case ucParagraphElements :
    {
      store = store_paragraphs;
      break;
    } 
    case ucPoetryElements :
    {
      store = store_poetry;
      break;
    } 
    case ucTableElements :
    {
      store = store_tables;
      break;
    }
    case ucFootnotes :
    {
      store = store_footnotes;
      break;
    } 
    case ucCrossReferences :
    {
      store = store_crossreference;
      break;
    } 
    case ucExtendedStudyNotes :
    {
      store = store_study;
      break;
    } 
    case ucSpecialTextAndCharacterStyles :
    {
      store = store_special_styles;
      break;
    }
    case ucSpecialTextAndCharacterStyles2 :
    {
      store = store_special_styles2;
      break;
    }
    case ucPeripheralMaterials :
    {
      store = store_peripheral;
      break;
    } 
    case ucOtherElements :
    {
      store = store_other;
      break;
    }
  }
  return store;
}


void SelectStyleDialog::select_marker (const string& marker)
// Selects 'marker' in the listviews.
{
  // If the 'marker' is found in the store, select it. Unselect everything else
  select_marker (GTK_TREE_MODEL (store_identification), select_identification, treeview_identification, marker);
  select_marker (GTK_TREE_MODEL (store_introduction), select_introduction, treeview_introduction, marker);
  select_marker (GTK_TREE_MODEL (store_introduction2), select_introduction2, treeview_introduction2, marker);
  select_marker (GTK_TREE_MODEL (store_titles), select_titles, treeview_titles, marker);
  select_marker (GTK_TREE_MODEL (store_titles2), select_titles2, treeview_titles2, marker);
  select_marker (GTK_TREE_MODEL (store_chapters_verses), select_chapters_verses, treeview_chapters_verses, marker);
  select_marker (GTK_TREE_MODEL (store_paragraphs), select_paragraphs, treeview_paragraphs, marker);
  select_marker (GTK_TREE_MODEL (store_poetry), select_poetry, treeview_poetry, marker);
  select_marker (GTK_TREE_MODEL (store_tables), select_tables, treeview_tables, marker);
  select_marker (GTK_TREE_MODEL (store_footnotes), select_footnotes, treeview_footnotes, marker);
  select_marker (GTK_TREE_MODEL (store_crossreference), select_crossreference, treeview_crossreference, marker);
  select_marker (GTK_TREE_MODEL (store_study), select_study, treeview_study, marker);
  select_marker (GTK_TREE_MODEL (store_special_styles), select_special_styles, treeview_special_styles, marker);
  select_marker (GTK_TREE_MODEL (store_special_styles2), select_special_styles2, treeview_special_styles2, marker);
  select_marker (GTK_TREE_MODEL (store_peripheral), select_peripheral, treeview_peripheral, marker);
  select_marker (GTK_TREE_MODEL (store_other), select_other, treeview_other, marker);
}


void SelectStyleDialog::select_marker (GtkTreeModel *store, GtkTreeSelection * selection, GtkWidget * widget, const string& marker)
{
  // Unselect everything.
  gtk_tree_selection_unselect_all (selection);
  // Some variables.
  GtkTreeIter iter;
  gboolean valid;
  gint index = 0;
  // Get the first iter in the store.
  valid = gtk_tree_model_get_iter_first (store, &iter);
  while (valid) {
    // Walk through the list, reading each row
    gchar *str_data;
    // Make sure you terminate calls to gtk_tree_model_get() with a '-1' value.
    gtk_tree_model_get (store, &iter, 0, &str_data, -1);
    // If this is the marker we wish to display, select it, put the cursor on it,
    // and focus on the listview.
    if (str_data == marker) {
      gtk_tree_selection_select_iter (selection, &iter);
      GtkTreePath * path = NULL;
      path = gtk_tree_path_new_from_indices (index, -1);
      if (path) {
        gtk_tree_view_set_cursor (GTK_TREE_VIEW (widget), path, NULL, false);
      }
      gtk_tree_path_free (path);
      gtk_widget_grab_focus (widget);
    }
    g_free (str_data);
    valid = gtk_tree_model_iter_next (store, &iter);
    index++;
  }
}


void SelectStyleDialog::get_markers (GtkTreeModel *store, vector<string>& markers)
// Gets the list of markers in a store.
{
  // Clear the container we place the result in.
  markers.clear();
  // Some variables.
  GtkTreeIter iter;
  gboolean valid;  
  // Get the first iter in the store.
  valid = gtk_tree_model_get_iter_first (store, &iter);
  while (valid) {
    // Walk through the list, reading each row
    gchar *str_data;
    // Make sure you terminate calls to gtk_tree_model_get() with a '-1' value.
    gtk_tree_model_get (store, &iter, 0, &str_data, -1);
    // Store marker.
    markers.push_back(str_data);
    // Free memory.    
    g_free (str_data);
    // Next iterator available?
    valid = gtk_tree_model_iter_next (store, &iter);
  }
}


bool SelectStyleDialog::get_next_category (UsfmCategories& category, bool next)
// Takes 'category' and gets the next one (or previous one), if available.
// Returns whether succesful.
{
  bool success = false;
  int cat;
  cat = category;
  if (next) {
    if (category < ucOtherElements) {
      cat++;
      success = true;
    }
  } else {
    if (category > ucIdentificationInformation) {
      cat--;
      success = true;
    }
  }
  category = UsfmCategories (cat);
  return success;
}
