/*
** Copyright (C) 2003-2005 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <gtk/gtk.h>
#include "dialogunicode.h"
#include "utilities.h"
#include "bible.h"


#define TEMPFILE "bibledit.unicode"
#define TEMPFILE2 "bibledit.unicode2"


void unicode_convert (const string& path, const string& encoding)
{
  // The command to find out about the current encoding, whether it would work.
  string command;
  command = "iconv -f '";
  command.append (encoding);
  command.append ("' -t UTF-8 '");
  command.append (path);
  command.append ("' > /dev/null 2>&1");
  int shellresult = system (command.c_str());
  // Make conversion.
  if (shellresult == 0) {
    // Conversion succeeded: make the conversion using iconv.
    command = "iconv -f ";
    command.append (encoding);
    command.append (" -t UTF-8");
  } else {
    // Conversion did not succeed: rescue whatever strings are there.
    command = "strings --bytes=2";
  }
  command.append (" '");
  command.append (path);
  command.append ("' > ");
  command.append (temporary_file (TEMPFILE2));
  system (command.c_str ());
  // Copy temporal file back to original file.
  command = "cp '";
  command.append (temporary_file (TEMPFILE2));
  command.append ("' '");
  command.append (path);
  command.append ("'");
  system (command.c_str());
}


UnicodeDialog::UnicodeDialog (const string & filetoconvert)
{
  // Store variabeles
  filename = filetoconvert;
  
  unicodedialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (unicodedialog), "Convert to UTF-8 Unicode");
  gtk_window_set_position (GTK_WINDOW (unicodedialog), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_modal (GTK_WINDOW (unicodedialog), TRUE);
  gtk_window_set_default_size (GTK_WINDOW (unicodedialog), 480, 480);
  // Set the type to a splash screen, so that the window manager does not show
  // a close button. Ideally the delete_event handler should prevent the window
  // from closing, but for one or another reason this does not work.
  // This splash screen hack therefore, is needed to prevent the user from closing
  // the window.
  gtk_window_set_type_hint (GTK_WINDOW (unicodedialog), GDK_WINDOW_TYPE_HINT_DIALOG);

  dialog_vbox1 = GTK_DIALOG (unicodedialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  // First informative label
  label1 = gtk_label_new (
    "The book you are now opening is not in the right Unicode format.\n"
    "It needs to be converted before use.\n\n"
    "Here is a list of possible encodings\n"
    "If one is selected, it will show the result as well\n"
    "When you're happy with the result, press OK.\n"
    "The book will then be converted.");
    
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (vbox1), label1, FALSE, FALSE, 4);
  gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);

  combo1 = gtk_combo_new ();
  gtk_widget_show (combo1);
  gtk_box_pack_start (GTK_BOX (vbox1), combo1, FALSE, FALSE, 9);

  // Get the whole list of possible encodings.
  vector <string> encodings;
  FILE *stream;
  string s = "iconv --list";
  stream = popen (s.c_str (), "r");
  char buf[1024];
  while (fgets (buf, sizeof (buf), stream)) {
    s = buf;
    s = trim (s);
    if (g_str_has_suffix (s.c_str(), "//"))
      s = s.substr (0, s.length () - 2);
    encodings.push_back (s);
  }
  pclose (stream);

  GList *combo1_items = NULL;
  for (unsigned int i = 0; i < encodings.size(); i++)
    combo1_items = g_list_append (combo1_items, (gpointer) encodings[i].c_str());
  gtk_combo_set_popdown_strings (GTK_COMBO (combo1), combo1_items);
  g_list_free (combo1_items);

  combo_entry1 = GTK_COMBO (combo1)->entry;
  gtk_widget_show (combo_entry1);
  gtk_editable_set_editable (GTK_EDITABLE (combo_entry1), FALSE);
  gtk_entry_set_activates_default (GTK_ENTRY (combo_entry1), TRUE);

  // Display the first entry.
  gtk_entry_set_text (GTK_ENTRY (combo_entry1), encodings[0].c_str());

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow1);
  gtk_box_pack_start (GTK_BOX (vbox1), scrolledwindow1, TRUE, TRUE, 2);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  textview1 = gtk_text_view_new ();
  gtk_widget_show (textview1);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), textview1);
  gtk_text_view_set_editable (GTK_TEXT_VIEW (textview1), FALSE);
  gtk_text_view_set_pixels_below_lines (GTK_TEXT_VIEW (textview1), 2);
  gtk_text_view_set_pixels_inside_wrap (GTK_TEXT_VIEW (textview1), 2);

  dialog_action_area1 = GTK_DIALOG (unicodedialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  okbutton1 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (unicodedialog), okbutton1, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton1, GTK_CAN_DEFAULT);

  g_signal_connect ((gpointer) combo_entry1, "changed",
                    G_CALLBACK (unicodedialog_on_combo_entry1_changed),
                    gpointer(this));

  gtk_widget_grab_focus (combo_entry1);
  gtk_widget_grab_default (okbutton1);

  // Update GUI.
  on_combo_changed ();
}


UnicodeDialog::~UnicodeDialog ()
{
  unlink (temporary_file (TEMPFILE).c_str ());
  gtk_widget_destroy (unicodedialog);
}


int UnicodeDialog::run ()
{
  return gtk_dialog_run (GTK_DIALOG (unicodedialog));
}


void
UnicodeDialog::on_combo_changed ()
{
  /*
     If the user selects an encoding, this will show the example as well.
   */
  // Convert the text to a temporary file using the selected encoding.
  encoding = gtk_entry_get_text (GTK_ENTRY (combo_entry1));
  // Copy the file to convert to a temporal location.
  string command;
  string tempfile;
  tempfile = temporary_file (TEMPFILE);
  command = "cp --force ";
  command.append (filename);
  command.append (" '");
  command.append (tempfile);
  command.append ("'");
  system (command.c_str());
  // Convert the file.
  unicode_convert (tempfile, encoding);
  // Display the result.
  gchar *contents;
  g_file_get_contents (tempfile.c_str(), &contents, NULL, NULL);
  gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview1)), contents, -1);
  g_free (contents);
}


void UnicodeDialog::unicodedialog_on_combo_entry1_changed  (GtkEditable     *editable,
                                        gpointer         user_data)
{
  ((UnicodeDialog *) user_data)->on_combo_changed();
}
