/*
** Copyright (C) 2003-2005 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "project.h"
#include "utilities.h"
#include "config.h"
#include <glib.h>
#include "directories.h"
#include "stylesheetutils.h"
#include "constants.h"
#include "gwrappers.h"


Project::Project (Configuration * configuration)
{
  projectname = configuration->project;
  myconfiguration = configuration;
  project_load ();
}


Project::Project (const string& project)
/*
Note: This second constructor is only for convenience. It is not as good as 
the first type, neither does it write changes to disc.
*/
{
  projectname = project;
  myconfiguration = NULL;
  project_load ();
}


Project::~Project ()
{
  if (myconfiguration) {
    // If the object goes out of scope, it saves the project settings.
    vector < string > output;
    output.push_back (VERSION);
    output.push_back ("There is no need for editing this file, because it is maintained by bibledit");
    output.push_back (font);
    output.push_back (mystylesheet);
    try
    {
      write_lines (filename, output);
    }
    catch (exception & ex)
    { 
      cerr << ex.what () << endl;
    }
  }
}


void Project::project_load ()
{
  // Read the project file, if there is any. Add the bits that are missing.
  filename = gw_build_filename (directories_get_projects(), projectname, "projectsettings");
  ReadText rc (filename, true);
  for (unsigned int i = 0; i <= 1000; i++) {
    switch (i) {
      case 2:
      {
        if (i < rc.lines.size ()) {
          font = rc.lines[i];
        }
        else {
          if (myconfiguration)
            font = myconfiguration->editor_font_name;
        }
        break;
      }
      case 3:
      {
        if (i < rc.lines.size ()) {
          mystylesheet = rc.lines[i];
        }
        else
        {
          mystylesheet = "Standard";
        }
        break;
      }
    }
  }
}


string Project::get_stylesheet ()
// Returns the stylesheet, checks it.
{
  string checked_sheet (mystylesheet);
  // See whether our stylesheet exists.
  vector<string> stylesheets;
  stylesheet_get_ones_available (stylesheets);
  set<string> sheets (stylesheets.begin(), stylesheets.end());
  // Sheet is there? Fine.
  if (sheets.find (checked_sheet) != sheets.end());
  // Sheets is not there - take Standard, if it's around.
  else if (sheets.find (STANDARDSHEET) != sheets.end())
    checked_sheet = STANDARDSHEET;
  // Else take first sheet in the list.
  else
    checked_sheet = stylesheets[0];
  // Update internal data.
  mystylesheet = checked_sheet;
  return checked_sheet;  
}


void Project::set_stylesheet (const string& stylesheet)
{
  mystylesheet = stylesheet;
}
