/*
** Copyright (C) 2003-2005 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

#ifndef INCLUDED_USFM_H
#define INCLUDED_USFM_H

#include "libraries.h"
#include "style.h"
#include "configuration.h"


class Usfm
{
  friend class UsfmInlineMarkers;
public:
  Usfm (const string& stylesheet);
  ~Usfm ();
  bool is_identifier (const string& usfm);
  bool is_comment (const string& usfm);
  bool is_running_header (const string& usfm);
  bool is_starting_paragraph (const string& usfm);
  bool is_inline_text (const string& usfm);
  bool is_chapter_number (const string& usfm);
  bool is_verse_number (const string& usfm);
  double fontsize ();
  int fontpercentage ();
  string italic ();
  string bold ();
  string underline ();
  string smallcaps ();
  bool superscript ();
  string justification ();
  double spacebefore ();
  double spaceafter ();
  double leftmargin ();
  double rightmargin ();
  double firstlineindent ();
  bool spancolumns ();
private:
  vector<Style> styles;
  string last_marker;
  int last_style_pointer;
  int get_style_pointer (const string& usfm);
};


bool usfm_is_id (const string& marker);
bool usfm_is_chapter (const string& marker);
bool usfm_is_verse (const string& marker);


#endif
