/*
** Copyright (C) 2003-2005 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

#ifndef INCLUDED_USFMTOOLS_H
#define INCLUDED_USFMTOOLS_H


#include "libraries.h"
#include "ustring.h"
#include "usfm.h"
#include "xmlfo-utils.h"


enum UsfmCategories {ucIdentificationInformation, 
                     ucIntroductionElements,
                     ucIntroductionElements2,
                     ucTitlesHeadingsAndLabels,
                     ucTitlesHeadingsAndLabels2,
                     ucChaptersAndVerses, 
                     ucParagraphElements, 
                     ucPoetryElements, 
                     ucTableElements,
                     ucFootnotes, 
                     ucCrossReferences, 
                     ucExtendedStudyNotes, 
                     ucSpecialTextAndCharacterStyles,
                     ucSpecialTextAndCharacterStyles2,
                     ucPeripheralMaterials, 
                     ucOtherElements};

ustring usfm_extract_marker (ustring & line);
int usfm_search_marker_from (const ustring & line, unsigned int from);
unsigned int usfm_search_marker_from_length (const ustring & line, unsigned int from);
string get_id_from_file (const string & filename);
void split_line_into_usfm_and_text (const ustring& line, ustring& marker, ustring& text);
void usfm_handle_inline_text (ustring& line, UsfmInlineMarkers * inline_markers, XmlFoBlock * block);
void usfm_categorize_markers (vector<string>& markers, vector<UsfmCategories>& categories);


class UsfmInlineMarkers
{
public:
  UsfmInlineMarkers (const Usfm& usfm);
  ~UsfmInlineMarkers ();
  vector<string> opening_markers;
  vector<string> closing_markers;
  string opening_xslfo_markup (XmlFoBlock * block, int index);
  string closing_xslfo_markup;
private:
  vector<double> fontpercentage;
  vector<string> italic;
  vector<string> bold;
  vector<string> underline;
  vector<string> smallcaps;
  vector<bool> superscript;
};


#endif
