/*
** Copyright (C) 2003-2005 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "utilities.h"
#include <libgen.h>
#include <glib.h>
#include <config.h>
#include "xep.h"
#include "java.h"
#include "gwrappers.h"
#include "gtkwrappers.h"


/*
As FOP sometimes fails to properly format a Scripture, the commercially 
available proprietary XEP is supported too.
*/


string modified_xep_command (Configuration * configuration)
{
  // Return the command to run xep. This command has been derived from script
  // xep that comes with the program. The amount of memory assigned to Java
  // has been increased in this command, to allow XEP to format a whole project.
  string returnvalue;
  string command;
  command = "JAVA_HOME=\"";
  command.append (configuration->java_home);
  command.append ("\"; XEP_HOME=\"");
  command.append (configuration->xep_home);
  command.append ("\"; CP=\"$JAVA_HOME/lib/tools.jar:$XEP_HOME/lib/xep.jar:$XEP_HOME/lib/saxon.jar:$XEP_HOME/lib/xt.jar\"; \"$JAVA_HOME/bin/java\" -Xmx256M -classpath \"$CP\" \"-Dcom.renderx.xep.CONFIG=$XEP_HOME/xep.xml\" com.renderx.xep.XSLDriver \"$@\" ");
  return command;
}


string standard_xep_command (Configuration * configuration)
{
  // Return the command to run script xep, if found.
  string returnvalue;
  string command;
  command = "which ";
  string xep = gw_build_filename (configuration->xep_home, "xep");
  command.append (xep);
  if (system (command.c_str()) != 0)
    xep.clear();
  return xep;
}


bool xep_present (GtkWidget * window, Configuration * configuration)
// Returns whether the script to run xep can be found.
// Gives message if not found.
{
  bool present = false;
  if (!standard_xep_command(configuration).empty())
    present = true;
  
  if (!present) {
    string message;
    message = "XEP was not found.\n";
    message.append ("XEP is set as your formatter of choice.\n");
    message.append ("It is required for printing.\n");
    message.append ("Install XEP first.\n");
    message.append ("For more information see menu Preferences->Formatter.");
    gtkw_dialog_info (window, message);
  }
  return present;
}


int xep_convert_to_pdf (Configuration * configuration, const string& xslfo, const string& pdf)
// Converts file "xslfo" to file "pdf", and returns the result.
{
  string command;
  command = modified_xep_command(configuration);
  command.append (" -valid -xml '");
  command.append (xslfo);
  command.append ("' -out '");
  command.append (pdf);
  command.append ("'");
  cout << command << endl;
  int result;
  result = system (command.c_str());
  return result;
}
